@@ -2,7 +2,7 @@
 #
 # Automated Testing Framework (atf)
 #
-# Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
+# Copyright (c) 2007 The NetBSD Foundation, Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -61,9 +61,9 @@ check_tool() {
     prog=${2}
     name=${3}
 
-    ver=$(${prog} --version | head -n 1 | cut -d ' ' -f 4-)
+    ver=$(${prog} --version | head -n 1 | cut -d ' ' -f 4)
 
-    if grep "^\\* ${name} ${ver}$" ${readme} >/dev/null; then
+    if grep "\\* ${name} ${ver}" ${readme} >/dev/null; then
         true
     else
         warn "Incorrect version of ${name}"
