@@ -64,7 +64,18 @@
     } \
     ATF_TEST_CASE_BODY(name) \
     { \
-        build_check_cxx_o(*this, sfile, failmsg); \
+        build_check_cxx_o(*this, sfile, failmsg, true);      \
+    }
+
+#define BUILD_TC_FAIL(name, sfile, descr, failmsg) \
+    ATF_TEST_CASE(name); \
+    ATF_TEST_CASE_HEAD(name) \
+    { \
+        set_md_var("descr", descr); \
+    } \
+    ATF_TEST_CASE_BODY(name) \
+    { \
+        build_check_cxx_o(*this, sfile, failmsg, false);      \
     }
 
 namespace atf {
@@ -74,7 +85,7 @@ class tc;
 }
 
 void header_check(const char*);
-void build_check_cxx_o(const atf::tests::tc&, const char*, const char*);
+void build_check_cxx_o(const atf::tests::tc&, const char*, const char*, bool);
 atf::fs::path get_process_helpers_path(const atf::tests::tc&);
 bool grep_file(const char*, const char*);
 bool grep_string(const std::string&, const char*);
