@@ -289,6 +289,7 @@ ATF_TEST_CASE_HEAD(pass)
 }
 ATF_TEST_CASE_BODY(pass)
 {
+    ATF_TEST_CASE_USE(h_pass);
     run_h_tc< ATF_TEST_CASE_NAME(h_pass) >();
     ATF_REQUIRE(grep_file("result", "^passed"));
     ATF_REQUIRE(atf::fs::exists(atf::fs::path("before")));
@@ -302,6 +303,7 @@ ATF_TEST_CASE_HEAD(fail)
 }
 ATF_TEST_CASE_BODY(fail)
 {
+    ATF_TEST_CASE_USE(h_fail);
     run_h_tc< ATF_TEST_CASE_NAME(h_fail) >();
     ATF_REQUIRE(grep_file("result", "^failed: Failed on purpose"));
     ATF_REQUIRE(atf::fs::exists(atf::fs::path("before")));
@@ -315,6 +317,7 @@ ATF_TEST_CASE_HEAD(skip)
 }
 ATF_TEST_CASE_BODY(skip)
 {
+    ATF_TEST_CASE_USE(h_skip);
     run_h_tc< ATF_TEST_CASE_NAME(h_skip) >();
     ATF_REQUIRE(grep_file("result", "^skipped: Skipped on purpose"));
     ATF_REQUIRE(atf::fs::exists(atf::fs::path("before")));
@@ -346,6 +349,7 @@ ATF_TEST_CASE_BODY(require)
 
         std::cout << "Checking with a " << t->cond << " value\n";
 
+        ATF_TEST_CASE_USE(h_require);
         run_h_tc< ATF_TEST_CASE_NAME(h_require) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -394,6 +398,7 @@ ATF_TEST_CASE_BODY(require_eq)
                   << " and expecting " << (t->ok ? "true" : "false")
                   << "\n";
 
+        ATF_TEST_CASE_USE(h_require_eq);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_eq) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -438,6 +443,7 @@ ATF_TEST_CASE_BODY(require_in)
         atf::tests::vars_map config;
         config["value"] = t->value;
 
+        ATF_TEST_CASE_USE(h_require_in);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_in) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -484,6 +490,7 @@ ATF_TEST_CASE_BODY(require_match)
                   << " and expecting " << (t->ok ? "true" : "false")
                   << "\n";
 
+        ATF_TEST_CASE_USE(h_require_match);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_match) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -528,6 +535,7 @@ ATF_TEST_CASE_BODY(require_not_in)
         atf::tests::vars_map config;
         config["value"] = t->value;
 
+        ATF_TEST_CASE_USE(h_require_not_in);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_not_in) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -573,6 +581,7 @@ ATF_TEST_CASE_BODY(require_throw)
         std::cout << "Checking with " << t->what << " and expecting "
                   << (t->ok ? "true" : "false") << "\n";
 
+        ATF_TEST_CASE_USE(h_require_throw);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_throw) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -607,8 +616,9 @@ ATF_TEST_CASE_BODY(require_throw_re)
     } *t, tests[] = {
         { "throw_int", false, "unexpected error" },
         { "throw_rt_match", true, NULL },
-        { "throw_rt_no_match", true, "threw.*runtime_error(baz foo bar a).*"
-          "does not match 'a foo bar baz'" },
+        { "throw_rt_no_match", false,
+          "threw.*runtime_error\\(baz foo bar a\\).*"
+          "does not match 'foo\\.\\*baz'" },
         { "no_throw_rt", false, "did not throw" },
         { NULL, false, NULL }
     };
@@ -623,7 +633,8 @@ ATF_TEST_CASE_BODY(require_throw_re)
         std::cout << "Checking with " << t->what << " and expecting "
                   << (t->ok ? "true" : "false") << "\n";
 
-        run_h_tc< ATF_TEST_CASE_NAME(h_require_throw) >(config);
+        ATF_TEST_CASE_USE(h_require_throw_re);
+        run_h_tc< ATF_TEST_CASE_NAME(h_require_throw_re) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
         if (t->ok) {
@@ -670,6 +681,7 @@ ATF_TEST_CASE_BODY(check_errno)
         atf::tests::vars_map config;
         config["what"] = t->what;
 
+        ATF_TEST_CASE_USE(h_check_errno);
         run_h_tc< ATF_TEST_CASE_NAME(h_check_errno) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -717,6 +729,7 @@ ATF_TEST_CASE_BODY(require_errno)
         atf::tests::vars_map config;
         config["what"] = t->what;
 
+        ATF_TEST_CASE_USE(h_require_errno);
         run_h_tc< ATF_TEST_CASE_NAME(h_require_errno) >(config);
 
         ATF_REQUIRE(atf::fs::exists(before));
@@ -747,6 +760,11 @@ BUILD_TC(use, "macros_hpp_test.cpp",
          "do not cause syntax errors when used",
          "Build of macros_hpp_test.cpp failed; some macros in "
          "atf-c++/macros.hpp are broken");
+BUILD_TC_FAIL(detect_unused_tests, "unused_test.cpp",
+         "Tests that defining an unused test case raises a warning (and thus "
+         "an error)",
+         "Build of unused_test.cpp passed; unused test cases are not properly "
+         "detected");
 
 // ------------------------------------------------------------------------
 // Main.
@@ -771,4 +789,5 @@ ATF_INIT_TEST_CASES(tcs)
     // Add the test cases for the header file.
     ATF_ADD_TEST_CASE(tcs, include);
     ATF_ADD_TEST_CASE(tcs, use);
+    ATF_ADD_TEST_CASE(tcs, detect_unused_tests);
 }
