@@ -1,7 +1,7 @@
 //
 // Automated Testing Framework (atf)
 //
-// Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
+// Copyright (c) 2007 The NetBSD Foundation, Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,24 +27,32 @@
 // IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 
+extern "C" {
+#include <sys/time.h>
+}
+
 #include <cstdlib>
 #include <fstream>
+#include <iomanip>
 #include <iostream>
 #include <memory>
+#include <sstream>
 #include <utility>
 #include <vector>
 
-#include "atf-c++/application.hpp"
-#include "atf-c++/fs.hpp"
-#include "atf-c++/sanity.hpp"
-#include "atf-c++/text.hpp"
-#include "atf-c++/ui.hpp"
+#include "atf-c/defs.h"
+
+#include "atf-c++/detail/application.hpp"
+#include "atf-c++/detail/fs.hpp"
+#include "atf-c++/detail/sanity.hpp"
+#include "atf-c++/detail/text.hpp"
+#include "atf-c++/detail/ui.hpp"
 
 #include "reader.hpp"
 
 typedef std::auto_ptr< std::ostream > ostream_ptr;
 
-ostream_ptr
+static ostream_ptr
 open_outfile(const atf::fs::path& path)
 {
     ostream_ptr osp;
@@ -57,6 +65,16 @@ open_outfile(const atf::fs::path& path)
     return osp;
 }
 
+static std::string
+format_tv(struct timeval* tv)
+{
+    std::ostringstream output;
+    output << static_cast< long >(tv->tv_sec) << '.'
+           << std::setfill('0') << std::setw(6)
+           << static_cast< long >(tv->tv_usec);
+    return output.str();
+}
+
 // ------------------------------------------------------------------------
 // The "writer" interface.
 // ------------------------------------------------------------------------
@@ -80,11 +98,12 @@ public:
     virtual void write_info(const std::string&, const std::string&) {}
     virtual void write_ntps(size_t) {}
     virtual void write_tp_start(const std::string&, size_t) {}
-    virtual void write_tp_end(const std::string&) {}
+    virtual void write_tp_end(struct timeval*, const std::string&) {}
     virtual void write_tc_start(const std::string&) {}
     virtual void write_tc_stdout_line(const std::string&) {}
     virtual void write_tc_stderr_line(const std::string&) {}
-    virtual void write_tc_end(const std::string&, const std::string&) {}
+    virtual void write_tc_end(const std::string&, struct timeval*,
+                              const std::string&) {}
     virtual void write_eof(void) {}
 };
 
@@ -115,7 +134,8 @@ public:
 
     virtual
     void
-    write_tp_start(const std::string& name, size_t ntcs)
+    write_tp_start(const std::string& name,
+                   size_t ntcs ATF_DEFS_ATTRIBUTE_UNUSED)
     {
         m_tpname = name;
         m_failed = false;
@@ -123,15 +143,17 @@ public:
 
     virtual
     void
-    write_tp_end(const std::string& reason)
+    write_tp_end(struct timeval* tv, const std::string& reason)
     {
+        const std::string timestamp = format_tv(tv);
+
         if (!reason.empty())
-            (*m_os) << "tp, " << m_tpname << ", bogus, " << reason
-                    << std::endl;
+            (*m_os) << "tp, " << timestamp << ", " << m_tpname << ", bogus, "
+                    << reason << "\n";
         else if (m_failed)
-            (*m_os) << "tp, " << m_tpname << ", failed" << std::endl;
+            (*m_os) << "tp, " << timestamp << ", "<< m_tpname << ", failed\n";
         else
-            (*m_os) << "tp, " << m_tpname << ", passed" << std::endl;
+            (*m_os) << "tp, " << timestamp << ", "<< m_tpname << ", passed\n";
     }
 
     virtual
@@ -143,12 +165,13 @@ public:
 
     virtual
     void
-    write_tc_end(const std::string& state, const std::string& reason)
+    write_tc_end(const std::string& state, struct timeval* tv,
+                 const std::string& reason)
     {
-        std::string str = "tc, " + m_tpname + ", " + m_tcname + ", " + state;
+        std::string str = m_tpname + ", " + m_tcname + ", " + state;
         if (!reason.empty())
             str += ", " + reason;
-        (*m_os) << str << std::endl;
+        (*m_os) << "tc, " << format_tv(tv) << ", " << str << "\n";
 
         if (state == "failed")
             m_failed = true;
@@ -174,15 +197,14 @@ class ticker_writer : public writer {
     size_t m_tcs_passed, m_tcs_failed, m_tcs_skipped, m_tcs_expected_failures;
     std::string m_tcname, m_tpname;
     std::vector< std::string > m_failed_tcs;
-    std::vector< std::string > m_expected_failures_tcs;
+    std::map< std::string, std::string > m_expected_failures_tcs;
     std::vector< std::string > m_failed_tps;
 
     void
     write_info(const std::string& what, const std::string& val)
     {
         if (what == "tests.root") {
-            (*m_os) << "Tests root: " << val << std::endl
-                    << std::endl;
+            (*m_os) << "Tests root: " << val << "\n\n";
         }
     }
 
@@ -208,12 +230,12 @@ class ticker_writer : public writer {
         (*m_os) << format_text(tp + " (" + to_string(m_curtp) +
                                "/" + to_string(m_ntps) + "): " +
                                to_string(ntcs) + " test cases")
-                << std::endl;
+                << "\n";
         (*m_os).flush();
     }
 
     void
-    write_tp_end(const std::string& reason)
+    write_tp_end(struct timeval* tv, const std::string& reason)
     {
         using atf::ui::format_text_with_tag;
 
@@ -224,10 +246,10 @@ class ticker_writer : public writer {
                                             "trust its results because "
                                             "of `" + reason + "'",
                                             m_tpname + ": ", false)
-                    << std::endl;
+                    << "\n";
             m_failed_tps.push_back(m_tpname);
         }
-        (*m_os) << std::endl;
+        (*m_os) << "[" << format_tv(tv) << "s]\n\n";
         (*m_os).flush();
 
         m_tpname.clear();
@@ -243,16 +265,19 @@ class ticker_writer : public writer {
     }
 
     void
-    write_tc_end(const std::string& state, const std::string& reason)
+    write_tc_end(const std::string& state, struct timeval* tv,
+                 const std::string& reason)
     {
         std::string str;
 
+        (*m_os) << "[" << format_tv(tv) << "s] ";
+
         if (state == "expected_death" || state == "expected_exit" ||
             state == "expected_failure" || state == "expected_signal" ||
             state == "expected_timeout") {
             str = "Expected failure: " + reason;
             m_tcs_expected_failures++;
-            m_expected_failures_tcs.push_back(m_tpname + ":" + m_tcname);
+            m_expected_failures_tcs[m_tpname + ":" + m_tcname] = reason;
         } else if (state == "failed") {
             str = "Failed: " + reason;
             m_tcs_failed++;
@@ -269,11 +294,30 @@ class ticker_writer : public writer {
         // XXX Wrap text.  format_text_with_tag does not currently allow
         // to specify the current column, which is needed because we have
         // already printed the tc's name.
-        (*m_os) << str << std::endl;
+        (*m_os) << str << '\n';
 
         m_tcname = "";
     }
 
+    static void
+    write_expected_failures(const std::map< std::string, std::string >& xfails,
+                            std::ostream& os)
+    {
+        using atf::ui::format_text;
+        using atf::ui::format_text_with_tag;
+
+        os << format_text("Test cases for known bugs:") << "\n";
+
+        for (std::map< std::string, std::string >::const_iterator iter =
+             xfails.begin(); iter != xfails.end(); iter++) {
+            const std::string& name = (*iter).first;
+            const std::string& reason = (*iter).second;
+
+            os << format_text_with_tag(reason, "    " + name + ": ", false)
+               << "\n";
+        }
+    }
+
     void
     write_eof(void)
     {
@@ -284,45 +328,36 @@ class ticker_writer : public writer {
 
         if (!m_failed_tps.empty()) {
             (*m_os) << format_text("Failed (bogus) test programs:")
-                    << std::endl;
+                    << "\n";
             (*m_os) << format_text_with_tag(join(m_failed_tps, ", "),
-                                            "    ", false) << std::endl
-                    << std::endl;
+                                            "    ", false) << "\n\n";
         }
 
         if (!m_expected_failures_tcs.empty()) {
-            (*m_os) << format_text("Test cases for known bugs:") << std::endl;
-            (*m_os) << format_text_with_tag(join(m_expected_failures_tcs, ", "),
-                                            "    ", false) << std::endl
-                    << std::endl;
+            write_expected_failures(m_expected_failures_tcs, *m_os);
+            (*m_os) << "\n";
         }
 
         if (!m_failed_tcs.empty()) {
-            (*m_os) << format_text("Failed test cases:") << std::endl;
+            (*m_os) << format_text("Failed test cases:") << "\n";
             (*m_os) << format_text_with_tag(join(m_failed_tcs, ", "),
-                                            "    ", false) << std::endl
-                    << std::endl;
+                                            "    ", false) << "\n\n";
         }
 
         (*m_os) << format_text("Summary for " + to_string(m_ntps) +
-                               " test programs:")
-                << std::endl;
+                               " test programs:") << "\n";
         (*m_os) << format_text_with_tag(to_string(m_tcs_passed) +
                                         " passed test cases.",
-                                        "    ", false)
-                << std::endl;
+                                        "    ", false) << "\n";
         (*m_os) << format_text_with_tag(to_string(m_tcs_failed) +
                                         " failed test cases.",
-                                        "    ", false)
-                << std::endl;
+                                        "    ", false) << "\n";
         (*m_os) << format_text_with_tag(to_string(m_tcs_expected_failures) +
                                         " expected failed test cases.",
-                                        "    ", false)
-                << std::endl;
+                                        "    ", false) << "\n";
         (*m_os) << format_text_with_tag(to_string(m_tcs_skipped) +
                                         " skipped test cases.",
-                                        "    ", false)
-                << std::endl;
+                                        "    ", false) << "\n";
     }
 
 public:
@@ -376,45 +411,46 @@ class xml_writer : public writer {
     void
     write_info(const std::string& what, const std::string& val)
     {
-        (*m_os) << "<info class=\"" << what << "\">" << val << "</info>"
-                << std::endl;
+        (*m_os) << "<info class=\"" << what << "\">" << val << "</info>\n";
     }
 
     void
-    write_tp_start(const std::string& tp, size_t ntcs)
+    write_tp_start(const std::string& tp,
+                   size_t ntcs ATF_DEFS_ATTRIBUTE_UNUSED)
     {
-        (*m_os) << "<tp id=\"" << attrval(tp) << "\">" << std::endl;
+        (*m_os) << "<tp id=\"" << attrval(tp) << "\">\n";
     }
 
     void
-    write_tp_end(const std::string& reason)
+    write_tp_end(struct timeval* tv, const std::string& reason)
     {
         if (!reason.empty())
-            (*m_os) << "<failed>" << elemval(reason) << "</failed>"
-                    << std::endl;
-        (*m_os) << "</tp>" << std::endl;
+            (*m_os) << "<failed>" << elemval(reason) << "</failed>\n";
+        (*m_os) << "<tp-time>" << format_tv(tv) << "</tp-time>";
+        (*m_os) << "</tp>\n";
     }
 
     void
     write_tc_start(const std::string& tcname)
     {
-        (*m_os) << "<tc id=\"" << attrval(tcname) << "\">" << std::endl;
+        (*m_os) << "<tc id=\"" << attrval(tcname) << "\">\n";
     }
 
     void
     write_tc_stdout_line(const std::string& line)
     {
-        (*m_os) << "<so>" << elemval(line) << "</so>" << std::endl;
+        (*m_os) << "<so>" << elemval(line) << "</so>\n";
     }
 
     void
     write_tc_stderr_line(const std::string& line)
     {
-        (*m_os) << "<se>" << elemval(line) << "</se>" << std::endl;
+        (*m_os) << "<se>" << elemval(line) << "</se>\n";
     }
 
     void
-    write_tc_end(const std::string& state, const std::string& reason)
+    write_tc_end(const std::string& state, struct timeval* tv,
+                 const std::string& reason)
     {
         std::string str;
 
@@ -422,37 +458,34 @@ class xml_writer : public writer {
             state == "expected_failure" || state == "expected_signal" ||
             state == "expected_timeout") {
             (*m_os) << "<" << state << ">" << elemval(reason)
-                    << "</" << state << ">" << std::endl;
+                    << "</" << state << ">\n";
         } else if (state == "passed") {
-            (*m_os) << "<passed />" << std::endl;
+            (*m_os) << "<passed />\n";
         } else if (state == "failed") {
-            (*m_os) << "<failed>" << elemval(reason)
-                    << "</failed>" << std::endl;
+            (*m_os) << "<failed>" << elemval(reason) << "</failed>\n";
         } else if (state == "skipped") {
-            (*m_os) << "<skipped>" << elemval(reason)
-                    << "</skipped>" << std::endl;
+            (*m_os) << "<skipped>" << elemval(reason) << "</skipped>\n";
         } else
             UNREACHABLE;
-        (*m_os) << "</tc>" << std::endl;
+        (*m_os) << "<tc-time>" << format_tv(tv) << "</tc-time>";
+        (*m_os) << "</tc>\n";
     }
 
     void
     write_eof(void)
     {
-        (*m_os) << "</tests-results>" << std::endl;
+        (*m_os) << "</tests-results>\n";
     }
 
 public:
     xml_writer(const atf::fs::path& p) :
         m_os(open_outfile(p))
     {
-        (*m_os) << "<?xml version=\"1.0\"?>" << std::endl
+        (*m_os) << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"
                 << "<!DOCTYPE tests-results PUBLIC "
                    "\"-//NetBSD//DTD ATF Tests Results 0.1//EN\" "
-                   "\"http://www.NetBSD.org/XML/atf/tests-results.dtd\">"
-                << std::endl
-                << std::endl
-                << "<tests-results>" << std::endl;
+                   "\"http://www.NetBSD.org/XML/atf/tests-results.dtd\">\n\n"
+                   "<tests-results>\n";
     }
 };
 
@@ -496,11 +529,11 @@ class converter : public atf::atf_report::atf_tps_reader {
     }
 
     void
-    got_tp_end(const std::string& reason)
+    got_tp_end(struct timeval* tv, const std::string& reason)
     {
         for (outs_vector::iterator iter = m_outs.begin();
              iter != m_outs.end(); iter++)
-            (*iter)->write_tp_end(reason);
+            (*iter)->write_tp_end(tv, reason);
     }
 
     void
@@ -528,11 +561,12 @@ class converter : public atf::atf_report::atf_tps_reader {
     }
 
     void
-    got_tc_end(const std::string& state, const std::string& reason)
+    got_tc_end(const std::string& state, struct timeval* tv,
+               const std::string& reason)
     {
         for (outs_vector::iterator iter = m_outs.begin();
              iter != m_outs.end(); iter++)
-            (*iter)->write_tc_end(state, reason);
+            (*iter)->write_tc_end(state, tv, reason);
     }
 
     void
@@ -636,6 +670,9 @@ atf_report::specific_options(void)
 int
 atf_report::main(void)
 {
+    if (m_argc > 0)
+        throw std::runtime_error("No arguments allowed");
+
     if (m_oflags.empty())
         m_oflags.push_back(fmt_path_pair("ticker", atf::fs::path("-")));
 
